@@ -9,9 +9,6 @@
 #undef PACKAGE_SOURCE_DIR
 #undef ENABLE_BONOBO
 #undef USING_OAF
-#undef ENABLE_FRGBA
-#undef ENABLE_RBUF
-#undef NEW_FONT_MODEL
 #undef NEW_RENDER
-#undef ARENA
+#undef ENABLE_AUTOTRACE
 
