@@ -1,12 +1,15 @@
 #define SP_QUICK_ALIGN_C
 
+#include <config.h>
 #include <math.h>
 #include <gnome.h>
 #include <glade/glade.h>
-#include "align.h"
 #include "../sodipodi.h"
+#include "../document.h"
 #include "../desktop-handles.h"
 #include "../sp-item-transform.h"
+#include "../selection.h"
+#include "align.h"
 
 /*
  * handler functions for quick align dialog
