@@ -26,6 +26,8 @@ static void sp_action_redo_chgattr (SPDocument *doc, SPAction *action);
 static void sp_action_redo_chgcontent (SPDocument *doc, SPAction *action);
 static void sp_action_redo_chgorder (SPDocument *doc, SPAction *action);
 
+static void sp_action_print_list (SPAction *action);
+
 static SPAction *sp_action_new (SPActionType type, const guchar *id);
 static void sp_action_free (SPAction *action);
 
@@ -43,18 +45,16 @@ sp_document_set_undo_sensitive (SPDocument *doc, gboolean sensitive)
 	doc->private->sensitive = sensitive;
 }
 
-void
-sp_document_done (SPDocument *doc)
+static void
+sp_document_private_done (SPDocument *doc, const guchar *key)
 {
 	g_assert (doc != NULL);
 	g_assert (SP_IS_DOCUMENT (doc));
 	g_assert (doc->private != NULL);
 	g_assert (doc->private->sensitive);
 
-	/* Clear modal undo key */
-	doc->private->key = NULL;
-
-	if (doc->private->actions == NULL) return;
+	/* Set modal undo key */
+	doc->actionkey = key;
 
 	g_assert (doc->private->redo == NULL);
 
@@ -73,10 +73,69 @@ sp_document_done (SPDocument *doc)
 	doc->private->actions = NULL;
 }
 
+void
+sp_document_done (SPDocument *doc)
+{
+	g_assert (doc != NULL);
+	g_assert (SP_IS_DOCUMENT (doc));
+	g_assert (doc->private != NULL);
+	g_assert (doc->private->sensitive);
+
+	if (!doc->private->actions) return;
+
+	sp_document_private_done (doc, NULL);
+}
+
 void
 sp_document_maybe_done (SPDocument *doc, const guchar *key)
 {
-	sp_document_done (doc);
+	if (!doc->private->actions) return;
+
+	if (key && doc->actionkey && !strcmp (key, doc->actionkey) && doc->private->undo) {
+		SPAction *last, *current;
+		current = doc->private->actions;
+		last = (SPAction *) doc->private->undo->data;
+		while (last && current) {
+			if (current->type != last->type) break;
+			if ((current->type != SP_ACTION_CHGATTR) && (current->type != SP_ACTION_CHGCONTENT)) break;
+			if (strcmp (current->id, last->id)) break;
+			current = current->next;
+			last = last->next;
+		}
+		if (!last && !current) {
+			/* Merge current into last */
+			current = doc->private->actions;
+			last = (SPAction *) doc->private->undo->data;
+			while (last && current) {
+				switch (current->type) {
+				case SP_ACTION_CHGATTR:
+					if (last->act.chgattr.newval) g_free (last->act.chgattr.newval);
+					last->act.chgattr.newval = current->act.chgattr.newval;
+					current->act.chgattr.newval = NULL;
+					break;
+				case SP_ACTION_CHGCONTENT:
+					if (last->act.chgcontent.newval) g_free (last->act.chgcontent.newval);
+					last->act.chgcontent.newval = current->act.chgcontent.newval;
+					current->act.chgcontent.newval = NULL;
+					break;
+				default:
+					g_warning ("file %s: line %d: Action %d should not be in merge list", __FILE__, __LINE__, current->type);
+					break;
+				}
+				current = current->next;
+				last = last->next;
+			}
+			/* fixme: IMHO NOP is handled by repr itself */
+			if (!sp_repr_attr (doc->private->rroot, "sodipodi:modified")) {
+				sp_repr_set_attr (doc->private->rroot, "sodipodi:modified", "true");
+			}
+			sp_action_free_list (doc->private->actions);
+			doc->private->actions = NULL;
+			return;
+		}
+	}
+
+	sp_document_private_done (doc, key);
 }
 
 void
@@ -88,7 +147,15 @@ sp_document_undo (SPDocument *doc)
 	g_assert (SP_IS_DOCUMENT (doc));
 	g_assert (doc->private != NULL);
 	g_assert (doc->private->sensitive);
-	g_assert (doc->private->actions == NULL);
+
+	/* Set modal undo key */
+	doc->actionkey = NULL;
+
+	if (doc->private->actions) {
+		g_warning ("Document Undo: Last operation did not flush cache");
+		sp_action_print_list (doc->private->actions);
+		return;
+	}
 
 	if (doc->private->undo == NULL) return;
 
@@ -121,6 +188,9 @@ sp_document_redo (SPDocument *doc)
 	g_assert (doc->private->sensitive);
 	g_assert (doc->private->actions == NULL);
 
+	/* Set modal undo key */
+	doc->actionkey = NULL;
+
 	if (doc->private->redo == NULL) return;
 
 	/* Get last action list */
@@ -513,6 +583,43 @@ sp_action_redo_chgorder (SPDocument *doc, SPAction *action)
 	sp_repr_change_order (object->repr, child->repr, (ref) ? ref->repr : NULL);
 }
 
+static void
+sp_action_print_list (SPAction *action)
+{
+	while (action) {
+		g_print ("SPAction: Id %s\n", action->id);
+		switch (action->type) {
+		case SP_ACTION_ADD:
+			g_print ("SPAction: Add ref %s\n", action->act.add.ref);
+			break;
+		case SP_ACTION_DEL:
+			g_print ("SPAction: Del ref %s\n", action->act.del.ref);
+			break;
+		case SP_ACTION_CHGATTR:
+			g_print ("SPAction: ChgAttr %s: %s -> %s\n",
+				 g_quark_to_string (action->act.chgattr.key),
+				 action->act.chgattr.oldval,
+				 action->act.chgattr.newval);
+			break;
+		case SP_ACTION_CHGCONTENT:
+			g_print ("SPAction: ChgContent %s -> %s\n",
+				 action->act.chgcontent.oldval,
+				 action->act.chgcontent.newval);
+			break;
+		case SP_ACTION_CHGORDER:
+			g_print ("SPAction: ChgOrder %s: %s -> %s\n",
+				 action->act.chgorder.child,
+				 action->act.chgorder.oldref,
+				 action->act.chgorder.newref);
+			break;
+		default:
+			g_print ("SPAction: Invalid action type %d\n", action->type);
+			break;
+		}
+		action = action->next;
+	}
+}
+
 #define SP_ACTION_ALLOC_SIZE 256
 static SPAction *free_action = NULL;
 
