@@ -1,5 +1,6 @@
 #define SP_EVENT_CONTEXT_C
 
+#include <config.h>
 #include <gtk/gtkmenu.h>
 #include <gtk/gtkmenuitem.h>
 #include <libgnome/gnome-defs.h>
@@ -16,7 +17,7 @@
 #include "file.h"
 #include "interface.h"
 #include "selection-chemistry.h"
-
+#include "dialogs/desktop-properties.h"
 
 static void sp_event_context_class_init (SPEventContextClass * klass);
 static void sp_event_context_init (SPEventContext * event_context);
@@ -124,21 +125,30 @@ sp_event_context_private_root_handler (SPEventContext * event_context, GdkEvent
 	static ArtPoint s;
 	static gboolean panning = FALSE;
 	gint ret;
-
+	SPDesktop * desktop;
 	ret = FALSE;
 
+       	desktop = event_context->desktop;
+
 	switch (event->type) {
+	case GDK_2BUTTON_PRESS:
+	  sp_desktop_dialog ();
+	  break;
 	case GDK_BUTTON_PRESS:
 		switch (event->button.button) {
 		case 2:
 			s.x = event->button.x;
 			s.y = event->button.y;
 			panning = TRUE;
+			gnome_canvas_item_grab (GNOME_CANVAS_ITEM (desktop->acetate),
+						GDK_BUTTON_RELEASE_MASK | GDK_POINTER_MOTION_MASK | GDK_BUTTON_PRESS_MASK,
+						NULL, event->button.time);
+
 			ret = TRUE;
 			break;
 		case 3:
 			/* fixme: */
-			sp_event_root_menu_popup (NULL, NULL, event);
+			sp_event_root_menu_popup (desktop, NULL, event);
 			break;
 		default:
 			break;
@@ -153,6 +163,7 @@ sp_event_context_private_root_handler (SPEventContext * event_context, GdkEvent
 	case GDK_BUTTON_RELEASE:
 		if (event->button.button == 2) {
 			panning = FALSE;
+			gnome_canvas_item_ungrab (GNOME_CANVAS_ITEM (desktop->acetate), event->button.time);
 			ret = TRUE;
 		}
 		break;
@@ -178,8 +189,8 @@ sp_event_context_private_root_handler (SPEventContext * event_context, GdkEvent
 	    sp_event_context_set_rect(NULL);	    
 	    ret = TRUE;
 	    break;
-	  case GDK_F5: // F5 - ellipse context
-	    sp_event_context_set_ellipse(NULL);	    
+	  case GDK_F5: // F5 - arc context
+	    sp_event_context_set_arc(NULL);	    
 	    ret = TRUE;
 	    break;
 	  case GDK_F6: // F6 - frehand line context
@@ -310,7 +321,7 @@ sp_event_context_private_root_handler (SPEventContext * event_context, GdkEvent
 	    }
 	    break;
 	  case GDK_space: // Space - root menu
-	    sp_event_root_menu_popup (NULL, NULL, event);
+	    sp_event_root_menu_popup (desktop, NULL, event);
 	    ret= TRUE;
 	    break;
           }
@@ -326,14 +337,14 @@ sp_event_context_private_root_handler (SPEventContext * event_context, GdkEvent
 /* fixme: do context sensitive popup menu on items */
 
 static gint
-sp_event_context_private_item_handler (SPEventContext * event_context, SPItem * item, GdkEvent * event)
+sp_event_context_private_item_handler (SPEventContext *ctx, SPItem *item, GdkEvent *event)
 {
 	switch (event->type) {
 	case GDK_BUTTON_PRESS:
 		switch (event->button.button) {
 		case 3:
 			/* fixme: */
-			sp_event_root_menu_popup (NULL, item, event);
+			sp_event_root_menu_popup (ctx->desktop, item, event);
 			return TRUE;
 		default:
 			break;
@@ -399,11 +410,13 @@ set_event_location (SPDesktop * desktop, GdkEvent * event)
 
 	sp_desktop_w2d_xy_point (desktop, &p, event->button.x, event->button.y);
 	sp_desktop_set_position (desktop, p.x, p.y);
+	sp_desktop_coordinate_status (desktop, p.x, p.y, 0);
 }
 
 void
-sp_event_root_menu_popup (GtkWidget * widget, SPItem * item, GdkEvent * event)
+sp_event_root_menu_popup (SPDesktop *desktop, SPItem *item, GdkEvent *event)
 {
+#if 0
 	static GtkMenu * menu = NULL;
 	static GtkWidget * objitem = NULL;
 	GladeXML * xml;
@@ -427,7 +440,7 @@ sp_event_root_menu_popup (GtkWidget * widget, SPItem * item, GdkEvent * event)
 		m = gtk_menu_new ();
 		gtk_signal_connect_while_alive (GTK_OBJECT (item), "destroy",
 						GTK_SIGNAL_FUNC (sp_event_grab_item_destroy), objitem, GTK_OBJECT (m));
-		sp_item_menu (item, GTK_MENU (m));
+		sp_item_menu (item, desktop, GTK_MENU (m));
 		gtk_menu_item_set_submenu (GTK_MENU_ITEM (objitem), m);
 		gtk_widget_show (m);
 		gtk_widget_set_sensitive (objitem, TRUE);
@@ -435,15 +448,22 @@ sp_event_root_menu_popup (GtkWidget * widget, SPItem * item, GdkEvent * event)
 		gtk_menu_item_remove_submenu (GTK_MENU_ITEM (objitem));
 		gtk_widget_set_sensitive (objitem, FALSE);
 	}
+#else
+	GtkWidget *menu;
+
+	menu = sp_ui_generic_menu (SP_VIEW (desktop), item);
+	gtk_widget_show (menu);
+#endif
 
 	switch (event->type) {
 	case GDK_BUTTON_PRESS:
-	  gtk_menu_popup (menu, NULL, NULL, 0, NULL, event->button.button, event->button.time);
-	  break;
+		gtk_menu_popup (GTK_MENU (menu), NULL, NULL, 0, NULL, event->button.button, event->button.time);
+		break;
 	case GDK_KEY_PRESS:
-	  gtk_menu_popup (menu, NULL, NULL, 0, NULL, 0, event->key.time);
-	  break;
+		gtk_menu_popup (GTK_MENU (menu), NULL, NULL, 0, NULL, 0, event->key.time);
+		break;
 	default:
+		break;
 	}
 }
 
