@@ -23,12 +23,14 @@
 enum {
 	ARG_0,
 	ARG_SHAPE,
+	ARG_MODE,
 	ARG_ANCHOR,
 	ARG_SIZE,
 	ARG_FILLED,
 	ARG_FILL_COLOR,
 	ARG_STROKED,
-	ARG_STROKE_COLOR
+	ARG_STROKE_COLOR,
+	ARG_PIXBUF
 };
 
 
@@ -36,7 +38,6 @@ static void sp_ctrl_class_init (SPCtrlClass *klass);
 static void sp_ctrl_init (SPCtrl *ctrl);
 static void sp_ctrl_destroy (GtkObject *object);
 static void sp_ctrl_set_arg (GtkObject *object, GtkArg *arg, guint arg_id);
-static void sp_ctrl_get_arg (GtkObject *object, GtkArg *arg, guint arg_id);
 
 static void sp_ctrl_update (GnomeCanvasItem *item, double *affine, ArtSVP *clip_path, int flags);
 static void sp_ctrl_render (GnomeCanvasItem *item, GnomeCanvasBuf *buf);
@@ -76,8 +77,10 @@ sp_ctrl_class_init (SPCtrlClass *klass)
 	parent_class = gtk_type_class (gnome_canvas_item_get_type ());
 
 	gtk_object_add_arg_type ("SPCtrl::shape", GTK_TYPE_ENUM, GTK_ARG_READWRITE, ARG_SHAPE);
+	gtk_object_add_arg_type ("SPCtrl::mode", GTK_TYPE_ENUM, GTK_ARG_READWRITE, ARG_MODE);
 	gtk_object_add_arg_type ("SPCtrl::anchor", GTK_TYPE_ANCHOR_TYPE, GTK_ARG_READWRITE, ARG_ANCHOR);
 	gtk_object_add_arg_type ("SPCtrl::size", GTK_TYPE_DOUBLE, GTK_ARG_READWRITE, ARG_SIZE);
+	gtk_object_add_arg_type ("SPCtrl::pixbuf", GTK_TYPE_POINTER, GTK_ARG_READWRITE, ARG_PIXBUF);
 	gtk_object_add_arg_type ("SPCtrl::filled", GTK_TYPE_BOOL, GTK_ARG_READWRITE, ARG_FILLED);
 	gtk_object_add_arg_type ("SPCtrl::fill_color", GTK_TYPE_INT, GTK_ARG_READWRITE, ARG_FILL_COLOR);
 	gtk_object_add_arg_type ("SPCtrl::stroked", GTK_TYPE_BOOL, GTK_ARG_READWRITE, ARG_STROKED);
@@ -85,7 +88,6 @@ sp_ctrl_class_init (SPCtrlClass *klass)
 
 	object_class->destroy = sp_ctrl_destroy;
 	object_class->set_arg = sp_ctrl_set_arg;
-	object_class->get_arg = sp_ctrl_get_arg;
 
 	item_class->update = sp_ctrl_update;
 	item_class->render = sp_ctrl_render;
@@ -96,16 +98,20 @@ static void
 sp_ctrl_init (SPCtrl *ctrl)
 {
 	ctrl->shape = SP_CTRL_SHAPE_SQUARE;
+	ctrl->mode = SP_CTRL_MODE_COLOR;
 	ctrl->anchor = GTK_ANCHOR_CENTER;
 	ctrl->span = 3;
 	ctrl->defined = TRUE;
 	ctrl->shown = FALSE;
+	ctrl->build = FALSE;
 	ctrl->filled = 1;
 	ctrl->stroked = 0;
 	ctrl->fill_color = 0x000000ff;
 	ctrl->stroke_color = 0x000000ff;
 
 	ctrl->box.x0 = ctrl->box.y0 = ctrl->box.x1 = ctrl->box.y1 = 0;
+	ctrl->cache = NULL;
+	ctrl->pixbuf = NULL;
 }
 
 static void
@@ -127,6 +133,7 @@ sp_ctrl_set_arg (GtkObject *object, GtkArg *arg, guint arg_id)
 {
 	GnomeCanvasItem *item;
 	SPCtrl *ctrl;
+	GdkPixbuf * pixbuf = NULL;
 
 	item = GNOME_CANVAS_ITEM (object);
 	ctrl = SP_CTRL (object);
@@ -134,54 +141,56 @@ sp_ctrl_set_arg (GtkObject *object, GtkArg *arg, guint arg_id)
 	switch (arg_id) {
 	case ARG_SHAPE:
 		ctrl->shape = GTK_VALUE_ENUM (*arg);
+		ctrl->build = FALSE;
+		gnome_canvas_item_request_update (item);
+		break;
+	case ARG_MODE:
+		ctrl->mode = GTK_VALUE_ENUM (*arg);
+		ctrl->build = FALSE;
 		gnome_canvas_item_request_update (item);
 		break;
 	case ARG_ANCHOR:
 		ctrl->anchor = GTK_VALUE_ENUM (*arg);
+		ctrl->build = FALSE;
 		gnome_canvas_item_request_update (item);
 		break;
 	case ARG_SIZE:
 		ctrl->span = (gint) ((GTK_VALUE_DOUBLE (*arg) - 1.0) / 2.0 + 0.5);
 		ctrl->defined = (ctrl->span > 0);
+		ctrl->build = FALSE;
 		gnome_canvas_item_request_update (item);
 		break;
 	case ARG_FILLED:
 		ctrl->filled = GTK_VALUE_BOOL (*arg);
+		ctrl->build = FALSE;
 		gnome_canvas_item_request_update (item);
 		break;
 	case ARG_FILL_COLOR:
 		ctrl->fill_color = GTK_VALUE_INT (*arg);
+		ctrl->build = FALSE;
 		gnome_canvas_item_request_update (item);
 		break;
 	case ARG_STROKED:
 		ctrl->stroked = GTK_VALUE_BOOL (*arg);
+		ctrl->build = FALSE;
 		gnome_canvas_item_request_update (item);
 		break;
 	case ARG_STROKE_COLOR:
 		ctrl->stroke_color = GTK_VALUE_INT (*arg);
+		ctrl->build = FALSE;
 		gnome_canvas_item_request_update (item);
 		break;
-	default:
-		break;
-	}
-}
-
-static void
-sp_ctrl_get_arg (GtkObject *object, GtkArg *arg, guint arg_id)
-{
-	SPCtrl *ctrl;
-
-	ctrl = SP_CTRL (object);
-
-	switch (arg_id) {
-	case ARG_ANCHOR:
-		GTK_VALUE_ENUM (*arg) = ctrl->anchor;
-		break;
-	case ARG_SIZE:
-		GTK_VALUE_DOUBLE (*arg) = 2.0 * ctrl->span + 1.0;
+	case ARG_PIXBUF:
+	        pixbuf  = GTK_VALUE_POINTER (*arg);
+		if (gdk_pixbuf_get_has_alpha (pixbuf)) {
+			ctrl->pixbuf = pixbuf;
+		} else {
+			ctrl->pixbuf = gdk_pixbuf_add_alpha (pixbuf, FALSE, 0, 0, 0);
+			gdk_pixbuf_unref (pixbuf);
+		}
+		ctrl->build = FALSE;
 		break;
 	default:
-		arg->type = GTK_TYPE_INVALID;
 		break;
 	}
 }
@@ -205,8 +214,8 @@ sp_ctrl_update (GnomeCanvasItem *item, double *affine, ArtSVP *clip_path, int fl
 
 	if (!ctrl->defined) return;
 
-	x = (gint) affine[4] - ctrl->span;
-	y = (gint) affine[5] - ctrl->span;
+	x = (gint) (affine[4] + 0.5) - ctrl->span;
+	y = (gint) (affine[5] + 0.5) - ctrl->span;
 
 	switch (ctrl->anchor) {
 	case GTK_ANCHOR_N:
@@ -266,112 +275,208 @@ sp_ctrl_point (GnomeCanvasItem *item, double x, double y,
 	return 1e18;
 }
 
-#define set_channel(p,v,a) ((p) + (((((v) - (p)) * (a)) + 0x80) >> 8))
-
 static void
-draw_line (GnomeCanvasBuf *buf, gint x0, gint x1, gint y, guint8 r, guint8 g, guint8 b, guint8 a)
+sp_ctrl_build_cache (SPCtrl *ctrl)
 {
-	gint x;
-	guchar *p;
-
-	x0 = MAX (x0, buf->rect.x0);
-	x1 = MIN (x1 + 1, buf->rect.x1);
-
-	p = buf->buf + (y - buf->rect.y0) * buf->buf_rowstride + (x0 - buf->rect.x0) * 3;
-
-	for (x = x0; x < x1; x++) {
-		*p++ = set_channel (*p, (guint) r, (guint) a);
-		*p++ = set_channel (*p, (guint) g, (guint) a);
-		*p++ = set_channel (*p, (guint) b, (guint) a);
-	}
-}
-
-static void
-draw_point (GnomeCanvasBuf *buf, gint x, gint y, guint8 r, guint8 g, guint8 b, guint8 a)
-{
-	guchar *p;
-
-	if (x < buf->rect.x0) return;
-	if (x >= buf->rect.x1) return;
-
-	p = buf->buf + (y - buf->rect.y0) * buf->buf_rowstride + (x - buf->rect.x0) * 3;
-
-	*p++ = set_channel (*p, (guint) r, (guint) a);
-	*p++ = set_channel (*p, (guint) g, (guint) a);
-	*p++ = set_channel (*p, (guint) b, (guint) a);
-}
-
-static void
-sp_ctrl_render (GnomeCanvasItem *item, GnomeCanvasBuf *buf)
-{
-	SPCtrl *ctrl;
+        guchar * p, *q;
+	gint size, x, y, z, s, a, side, c;
 	guint8 fr, fg, fb, fa, sr, sg, sb, sa;
-
-	ctrl = SP_CTRL (item);
-
-	if (!ctrl->defined) return;
-	if ((!ctrl->filled) && (!ctrl->stroked)) return;
-
-	gnome_canvas_buf_ensure_buf (buf);
-	buf->is_bg = FALSE;
-
-	fr = (ctrl->fill_color >> 24) & 0xff;
-	fg = (ctrl->fill_color >> 16) & 0xff;
-	fb = (ctrl->fill_color >> 8) & 0xff;
-	fa = (ctrl->fill_color) & 0xff;
+	
+	if (ctrl->filled) {
+	       fr = (ctrl->fill_color >> 24) & 0xff;
+	       fg = (ctrl->fill_color >> 16) & 0xff;
+	       fb = (ctrl->fill_color >> 8) & 0xff;
+	       fa = (ctrl->fill_color) & 0xff;
+	} else { fr = 0x00; fg = 0x00; fb = 0x00; fa = 0x00; }
 	if (ctrl->stroked) {
 		sr = (ctrl->stroke_color >> 24) & 0xff;
 		sg = (ctrl->stroke_color >> 16) & 0xff;
 		sb = (ctrl->stroke_color >> 8) & 0xff;
 		sa = (ctrl->stroke_color) & 0xff;
-	} else {
-		sr = fr;
-		sg = fg;
-		sb = fb;
-		sa = fa;
-	}
+	} else { sr = fr; sg = fg; sb = fb; sa = fa; }
 
-	if (ctrl->shape == SP_CTRL_SHAPE_SQUARE) {
-		gint y0, y1, y;
-		y0 = MAX (ctrl->box.y0, buf->rect.y0);
-		y1 = MIN (ctrl->box.y1, buf->rect.y1 - 1);
-		if (y0 == ctrl->box.y0) {
-			draw_line (buf, ctrl->box.x0, ctrl->box.x1, y0, sr, sg, sb, sa);
-			y0 += 1;
+
+ 	side = (ctrl->span * 2 +1);
+	c = ctrl->span ;
+	if (ctrl->cache) g_free (ctrl->cache);
+	size = (side) * (side) * 4;
+	ctrl->cache = g_malloc (size);
+	if (side < 2) return;
+	
+	switch (ctrl->shape) {
+	case SP_CTRL_SHAPE_SQUARE: 
+		p = ctrl->cache;
+		for (x=0; x < side; x++) { *p++ = sr; *p++ = sg; *p++ = sb; *p++ = sa; }
+		for (y = 2; y < side; y++) {
+			*p++ = sr; *p++ = sg; *p++ = sb; *p++ = sa;
+			for (x=2; x < side; x++) { *p++ = fr; *p++ = fg; *p++ = fb; *p++ = fa; }
+			*p++ = sr; *p++ = sg; *p++ = sb; *p++ = sa;
 		}
-		if (y1 == ctrl->box.y1) {
-			draw_line (buf, ctrl->box.x0, ctrl->box.x1, y1, sr, sg, sb, sa);
-			y1 -= 1;
+		for (x=0; x < side; x++) { *p++ = sr; *p++ = sg; *p++ = sb; *p++ = sa; }
+		ctrl->build = TRUE;
+		break;
+	case SP_CTRL_SHAPE_DIAMOND:
+		p = ctrl->cache;
+		for (y = 0; y < side; y++) {
+			z = abs (c - y);
+			for (x = 0; x < z; x++) {*p++ = 0x00; *p++ = 0x00; *p++ = 0x00; *p++ = 0x00;}
+			*p++ = sr; *p++ = sg; *p++ = sb; *p++ = sa; x++; 
+			for (; x < side - z -1; x++) { *p++ = fr; *p++ = fg; *p++ = fb; *p++ = fa; }
+			if (z != c) {*p++ = sr; *p++ = sg; *p++ = sb; *p++ = sa; x++;} 
+			for (; x < side; x++) {*p++ = 0x00; *p++ = 0x00; *p++ = 0x00; *p++ = 0x00;}
 		}
-		for (y = y0; y <= y1; y++) {
-			draw_point (buf, ctrl->box.x0, y, sr, sg, sb, sa);
-			if (ctrl->filled) draw_line (buf, ctrl->box.x0 + 1, ctrl->box.x1 - 1, y, sr, sg, sb, sa);
-			draw_point (buf, ctrl->box.x1, y, sr, sg, sb, sa);
+		break;
+	case SP_CTRL_SHAPE_CIRCLE:
+		p = ctrl->cache;
+		q = p + size -1;
+		s = -1;
+		for (y = 0; y <= c ; y++) {
+			a = abs (c - y);
+			z = (gint)(0.0 + sqrt ((c+0.5)*(c+0.5) - a*a));
+			x = 0;
+			while (x < c-z) {
+				*p++ = 0x00; *p++ = 0x00; *p++ = 0x00; *p++ = 0x00;
+				*q-- = 0x00; *q-- = 0x00; *q-- = 0x00; *q-- = 0x00; 
+				x++;
+			}
+			do {
+				*p++ = sr; *p++ = sg; *p++ = sb; *p++ = sa;   
+				*q-- = sa; *q-- = sb; *q-- = sg; *q-- = sr;
+				x++;
+			} while (x < c-s);
+			while (x < MIN(c+s+1, c+z)) {
+				*p++ = fr;   *p++ = fg;   *p++ = fb;   *p++ = fa;   
+				*q-- = fa;   *q-- = fb;   *q-- = fg;   *q-- = fr;   
+				x++;
+			}
+			do {
+				*p++ = sr;   *p++ = sg;   *p++ = sb;   *p++ = sa;   
+				*q-- = sa; *q-- = sb; *q-- = sg; *q-- = sr;
+				x++;
+			} while (x <= c+z);
+			while (x < side) {
+				*p++ = 0x00; *p++ = 0x00; *p++ = 0x00; *p++ = 0x00; 
+				*q-- = 0x00; *q-- = 0x00; *q-- = 0x00; *q-- = 0x00; 
+				x++;
+			}
+			s = z;
 		}
-	} else if (ctrl->shape == SP_CTRL_SHAPE_DIAMOND) {
-		gint cx, cy, y0, y1, y;
-		cx = (ctrl->box.x0 + ctrl->box.x1) / 2;
-		cy = (ctrl->box.y0 + ctrl->box.y1) / 2;
-		y0 = MAX (ctrl->box.y0, buf->rect.y0);
-		y1 = MIN (ctrl->box.y1, buf->rect.y1 - 1);
-		if (y0 == ctrl->box.y0) {
-			draw_point (buf, cx, y0, sr, sg, sb, sa);
-			y0 += 1;
+		ctrl->build = TRUE;
+		break;
+	case SP_CTRL_SHAPE_CROSS:
+		p = ctrl->cache;
+		for (y = 0; y < side; y++) {
+			z = abs (c - y);
+			for (x = 0; x < c-z; x++) {*p++ = 0x00; *p++ = 0x00; *p++ = 0x00; *p++ = 0x00;}
+			*p++ = sr; *p++ = sg; *p++ = sb; *p++ = sa; x++; 
+			for (; x < c + z; x++) {*p++ = 0x00; *p++ = 0x00; *p++ = 0x00; *p++ = 0x00;}
+			if (z != 0) {*p++ = sr; *p++ = sg; *p++ = sb; *p++ = sa; x++;} 
+			for (; x < side; x++) {*p++ = 0x00; *p++ = 0x00; *p++ = 0x00; *p++ = 0x00;}
 		}
-		if (y1 == ctrl->box.y1) {
-			draw_point (buf, cx, y1, sr, sg, sb, sa);
-			y1 -= 1;
-		}
-		for (y = y0; y <= y1; y++) {
-			gint r, s;
-			r = (cy >= y) ? cy - y : y - cy;
-			s = ctrl->span - r;
-			draw_point (buf, cx - s, y, sr, sg, sb, sa);
-			if (ctrl->filled) draw_line (buf, cx - s + 1, cx + s - 1, y, sr, sg, sb, sa);
-			draw_point (buf, cx + s, y, sr, sg, sb, sa);
+		ctrl->build = TRUE;
+		break;
+	case SP_CTRL_SHAPE_BITMAP:
+		if (ctrl->pixbuf) {
+			guint r = gdk_pixbuf_get_rowstride (ctrl->pixbuf);
+			guint * pix;
+			q = gdk_pixbuf_get_pixels (ctrl->pixbuf);
+			p = ctrl->cache;
+			for (y = 0; y < side; y++){
+				for (x = 0; x < side; x++) {
+					pix = (gpointer)q + (y * r) + x * 4;
+					if ((*pix & 0xff000000) < 0x80000000) {
+						*p++ = 0x00;
+						*p++ = 0x00;
+						*p++ = 0x00;
+						*p++ = 0x00;
+					} else if (*pix < 0xff800000){
+						*p++ = sr;
+						*p++ = sg;
+						*p++ = sb;
+						*p++ = sa;
+					} else {
+						*p++ = fr;
+						*p++ = fg;
+						*p++ = fb;
+						*p++ = fa;
+					}
+				}}
+		} else { g_print ("control has no pixmap\n"); }
+		ctrl->build = TRUE;
+		break;
+	case SP_CTRL_SHAPE_IMAGE:
+		if (ctrl->pixbuf) {
+			guint r = gdk_pixbuf_get_rowstride (ctrl->pixbuf);
+			guchar * pix;
+			q = gdk_pixbuf_get_pixels (ctrl->pixbuf);
+			p = ctrl->cache;
+			for (y = 0; y < side; y++){
+				pix = q + (y * r);
+				for (x = 0; x < side; x++) {
+					*p++ = *pix++;
+					*p++ = *pix++;
+					*p++ = *pix++;
+					*p++ = *pix++;
+				}
+			}
+		} else { g_print ("control has no pixmap\n"); } 
+		ctrl->build = TRUE;
+		break;
+	default:
+		break;
+	}
+	
+}
+
+#define COMPOSEP11(fc,fa,bc) (((255 - (fa)) * (bc) + (fc) * 255 + 127) / 255)
+#define COMPOSEN11(fc,fa,bc) (((255 - (fa)) * (bc) + (fc) * (fa) + 127) / 255)
+
+static void
+sp_ctrl_render (GnomeCanvasItem *item, GnomeCanvasBuf *buf)
+{
+	SPCtrl *ctrl;
+	gint y0, y1, y, x0,x1,x;
+	guchar * p, * q, a;
+	
+	ctrl = SP_CTRL (item);
+	
+	if (!ctrl->defined) return;
+	if ((!ctrl->filled) && (!ctrl->stroked)) return;
+	
+	gnome_canvas_buf_ensure_buf (buf);
+	buf->is_bg = FALSE;
+	
+	// the control-image is rendered into ctrl->cache
+	if (!ctrl->build) sp_ctrl_build_cache (ctrl);
+	
+	// then we render from ctrl->cache
+	y0 = MAX (ctrl->box.y0, buf->rect.y0);
+	y1 = MIN (ctrl->box.y1, buf->rect.y1 - 1);
+	x0 = MAX (ctrl->box.x0, buf->rect.x0);
+	x1 = MIN (ctrl->box.x1, buf->rect.x1 - 1);
+	for (y = y0; y <= y1; y++) {
+		p = buf->buf +    (y - buf->rect.y0) * buf->buf_rowstride + (x0 - buf->rect.x0) * 3;
+		q = ctrl->cache + ((y - ctrl->box.y0) * (ctrl->span*2+1) + (x0 - ctrl->box.x0)) * 4;
+		for (x = x0; x <= x1; x++) {
+			a = *(q + 3);
+			switch (ctrl->mode) {
+			case SP_CTRL_MODE_COLOR:
+				p[0] = COMPOSEN11 (q[0], a, p[0]);
+				p[1] = COMPOSEN11 (q[1], a, p[1]);
+				p[2] = COMPOSEN11 (q[2], a, p[2]);
+				q += 4;
+				p += 3;
+				break;
+			case SP_CTRL_MODE_XOR:
+				p[0] = COMPOSEN11 (q[0], a, p[0] ^ q[0]);
+				p[1] = COMPOSEN11 (q[1], a, p[1] ^ q[1]);
+				p[2] = COMPOSEN11 (q[2], a, p[2] ^ q[2]);
+				q += 4;
+				p += 3;
+				break;
+			}
 		}
 	}
-
 	ctrl->shown = TRUE;
 }
 
