@@ -26,10 +26,12 @@ enum {
 	ARG_SIZE,
 	ARG_ANCHOR,
 	ARG_SHAPE,
+	ARG_MODE,
 	ARG_FILL, ARG_FILL_MOUSEOVER, ARG_FILL_DRAGGING,
 	ARG_STROKE, ARG_STROKE_MOUSEOVER, ARG_STROKE_DRAGGING,
 	ARG_IMAGE, ARG_IMAGE_MOUSEOVER, ARG_IMAGE_DRAGGING,
-	ARG_CURSOR, ARG_CURSOR_MOUSEOVER, ARG_CURSOR_DRAGGING
+	ARG_CURSOR, ARG_CURSOR_MOUSEOVER, ARG_CURSOR_DRAGGING,
+	ARG_PIXBUF
 };
 
 enum {
@@ -43,14 +45,8 @@ enum {
 	LAST_SIGNAL
 };
 
-static void sp_marshal_BOOL__POINTER_UINT (GtkObject * object,
-	GtkSignalFunc func,
-	gpointer func_data,
-	GtkArg * args);
-static void sp_marshal_DOUBLE__POINTER_UINT (GtkObject * object,
-	GtkSignalFunc func,
-	gpointer func_data,
-	GtkArg * args);
+static void sp_marshal_BOOL__POINTER_UINT (GtkObject * object, GtkSignalFunc func, gpointer func_data, GtkArg * args);
+static void sp_marshal_DOUBLE__POINTER_UINT (GtkObject * object, GtkSignalFunc func, gpointer func_data, GtkArg * args);
 
 static void sp_knot_class_init (SPKnotClass * klass);
 static void sp_knot_init (SPKnot * knot);
@@ -60,6 +56,7 @@ static void sp_knot_set_arg (GtkObject * object, GtkArg * arg, guint id);
 static void sp_knot_handler (GnomeCanvasItem * item, GdkEvent * event, gpointer data);
 static void sp_knot_set_flag (SPKnot * knot, guint flag, gboolean set);
 static void sp_knot_update_ctrl (SPKnot * knot);
+static void sp_knot_set_ctrl_state (SPKnot *knot);
 
 static GtkObjectClass * parent_class;
 static guint knot_signals[LAST_SIGNAL] = {0};
@@ -75,9 +72,7 @@ sp_knot_get_type (void)
 			sizeof (SPKnotClass),
 			(GtkClassInitFunc) sp_knot_class_init,
 			(GtkObjectInitFunc) sp_knot_init,
-			NULL, /* reserved_1 */
-			NULL, /* reserved_2 */
-			(GtkClassInitFunc) NULL
+			NULL, NULL, NULL
 		};
 		knot_type = gtk_type_unique (gtk_object_get_type (), &knot_info);
 	}
@@ -98,6 +93,7 @@ sp_knot_class_init (SPKnotClass * klass)
 	gtk_object_add_arg_type ("SPKnot::size", GTK_TYPE_UINT, GTK_ARG_WRITABLE, ARG_SIZE);
 	gtk_object_add_arg_type ("SPKnot::anchor", GTK_TYPE_ANCHOR_TYPE, GTK_ARG_WRITABLE, ARG_ANCHOR);
 	gtk_object_add_arg_type ("SPKnot::shape", GTK_TYPE_ENUM, GTK_ARG_WRITABLE, ARG_SHAPE);
+	gtk_object_add_arg_type ("SPKnot::mode", GTK_TYPE_ENUM, GTK_ARG_WRITABLE, ARG_MODE);
 	gtk_object_add_arg_type ("SPKnot::fill", GTK_TYPE_UINT, GTK_ARG_WRITABLE, ARG_FILL);
 	gtk_object_add_arg_type ("SPKnot::fill_mouseover", GTK_TYPE_UINT, GTK_ARG_WRITABLE, ARG_FILL_MOUSEOVER);
 	gtk_object_add_arg_type ("SPKnot::fill_dragging", GTK_TYPE_UINT, GTK_ARG_WRITABLE, ARG_FILL_DRAGGING);
@@ -110,6 +106,7 @@ sp_knot_class_init (SPKnotClass * klass)
 	gtk_object_add_arg_type ("SPKnot::cursor", GTK_TYPE_GDK_CURSOR_TYPE, GTK_ARG_WRITABLE, ARG_CURSOR);
 	gtk_object_add_arg_type ("SPKnot::cursor_mouseover", GTK_TYPE_GDK_CURSOR_TYPE, GTK_ARG_WRITABLE, ARG_CURSOR_MOUSEOVER);
 	gtk_object_add_arg_type ("SPKnot::cursor_dragging", GTK_TYPE_GDK_CURSOR_TYPE, GTK_ARG_WRITABLE, ARG_CURSOR_DRAGGING);
+	gtk_object_add_arg_type ("SPKnot::pixbuf", GTK_TYPE_POINTER, GTK_ARG_WRITABLE, ARG_PIXBUF);
 
 	knot_signals[EVENT] = gtk_signal_new ("event",
 		GTK_RUN_LAST,
@@ -173,6 +170,7 @@ sp_knot_init (SPKnot * knot)
 	knot->hx = knot->hy = 0.0;
 	knot->anchor = GTK_ANCHOR_CENTER;
 	knot->shape = SP_KNOT_SHAPE_SQUARE;
+	knot->mode = SP_KNOT_MODE_COLOR;
 
 	knot->fill [SP_KNOT_STATE_NORMAL] = 0x000000ff;
 	knot->fill [SP_KNOT_STATE_MOUSEOVER] = 0xff0000ff;
@@ -191,23 +189,26 @@ sp_knot_init (SPKnot * knot)
 	knot->cursor [SP_KNOT_STATE_DRAGGING] = NULL;
 
 	knot->saved_cursor = NULL;
+	knot->pixbuf = NULL;
 }
 
 static void
 sp_knot_destroy (GtkObject * object)
 {
 	SPKnot * knot;
-	gint i;
 
 	knot = (SPKnot *) object;
 
+	/* ungrab pointer if still grabbed by mouseover, find a different way */
+	if (gdk_pointer_is_grabbed) gdk_pointer_ungrab (0);
+
 	if (knot->item) gtk_object_destroy (GTK_OBJECT (knot->item));
-
-	for (i = 0; i < SP_KNOT_VISIBLE_STATES; i++) {
-		/* fixme: There should be ::unref somewhere */
-		if (knot->cursor[i]) gdk_cursor_destroy (knot->cursor[i]);
-	}
-
+	/*
+	  for (i = 0; i < SP_KNOT_VISIBLE_STATES; i++) {
+	  
+	  if (knot->cursor[i]) gdk_cursor_destroy (knot->cursor[i]);
+	  }
+	*/
 	if (((GtkObjectClass *) (parent_class))->destroy)
 		(* ((GtkObjectClass *) (parent_class))->destroy) (object);
 }
@@ -223,12 +224,16 @@ sp_knot_set_arg (GtkObject * object, GtkArg * arg, guint id)
 
 	switch (id) {
 	case ARG_SIZE:
-		knot->size = GTK_VALUE_UINT (* arg);
+		knot->size = GTK_VALUE_UINT (*arg);
 		break;
 	case ARG_ANCHOR:
-		knot->anchor = GTK_VALUE_ENUM (* arg);
+		knot->anchor = GTK_VALUE_ENUM (*arg);
 		break;
 	case ARG_SHAPE:
+		knot->shape = GTK_VALUE_ENUM (*arg);
+		break;
+	case ARG_MODE:
+		knot->mode = GTK_VALUE_ENUM (*arg);
 		break;
 	case ARG_FILL:
 		knot->fill[SP_KNOT_STATE_NORMAL] =
@@ -236,7 +241,8 @@ sp_knot_set_arg (GtkObject * object, GtkArg * arg, guint id)
 		knot->fill[SP_KNOT_STATE_DRAGGING] = GTK_VALUE_UINT (* arg);
 		break;
 	case ARG_FILL_MOUSEOVER:
-		knot->fill[SP_KNOT_STATE_MOUSEOVER] = GTK_VALUE_UINT (* arg);
+		knot->fill[SP_KNOT_STATE_MOUSEOVER] = 
+		knot->fill[SP_KNOT_STATE_DRAGGING] = GTK_VALUE_UINT (* arg);
 		break;
 	case ARG_FILL_DRAGGING:
 		knot->fill[SP_KNOT_STATE_DRAGGING] = GTK_VALUE_UINT (* arg);
@@ -244,40 +250,51 @@ sp_knot_set_arg (GtkObject * object, GtkArg * arg, guint id)
 	case ARG_STROKE:
 		knot->stroke[SP_KNOT_STATE_NORMAL] =
 		knot->stroke[SP_KNOT_STATE_MOUSEOVER] =
-		knot->stroke[SP_KNOT_STATE_DRAGGING] = GTK_VALUE_UINT (* arg);
+		knot->stroke[SP_KNOT_STATE_DRAGGING] = GTK_VALUE_UINT (*arg);
 		break;
 	case ARG_STROKE_MOUSEOVER:
-		knot->stroke[SP_KNOT_STATE_MOUSEOVER] = GTK_VALUE_UINT (* arg);
+		knot->stroke[SP_KNOT_STATE_MOUSEOVER] = 
+		knot->stroke[SP_KNOT_STATE_DRAGGING] = GTK_VALUE_UINT (*arg);
 		break;
 	case ARG_STROKE_DRAGGING:
-		knot->stroke[SP_KNOT_STATE_DRAGGING] = GTK_VALUE_UINT (* arg);
+		knot->stroke[SP_KNOT_STATE_DRAGGING] = GTK_VALUE_UINT (*arg);
 		break;
 	case ARG_IMAGE:
+		knot->image[SP_KNOT_STATE_NORMAL] =
+		knot->image[SP_KNOT_STATE_MOUSEOVER] =
+		knot->image[SP_KNOT_STATE_DRAGGING] = GTK_VALUE_POINTER (*arg);
+		break;
 	case ARG_IMAGE_MOUSEOVER:
+		knot->image[SP_KNOT_STATE_MOUSEOVER] = GTK_VALUE_POINTER (*arg);
+		break;
 	case ARG_IMAGE_DRAGGING:
+		knot->image[SP_KNOT_STATE_DRAGGING] = GTK_VALUE_POINTER (*arg);
 		break;
 	case ARG_CURSOR:
 		cursor = GTK_VALUE_BOXED (* arg);
 		for (i = 0; i < SP_KNOT_VISIBLE_STATES; i++) {
 			/* fixme: There should be ::unref somewhere */
-			if (knot->cursor[i]) gdk_cursor_destroy (knot->cursor[i]);
+			//if (knot->cursor[i]) gdk_cursor_destroy (knot->cursor[i]);
 			knot->cursor[i] = cursor;
 		}
 		break;
 	case ARG_CURSOR_MOUSEOVER:
 		cursor = GTK_VALUE_BOXED (* arg);
 		if (knot->cursor[SP_KNOT_STATE_MOUSEOVER]) {
-			gdk_cursor_destroy (knot->cursor[SP_KNOT_STATE_MOUSEOVER]);
+			//gdk_cursor_destroy (knot->cursor[SP_KNOT_STATE_MOUSEOVER]);
 		}
 		knot->cursor[SP_KNOT_STATE_MOUSEOVER] = cursor;
 		break;
 	case ARG_CURSOR_DRAGGING:
 		cursor = GTK_VALUE_BOXED (* arg);
 		if (knot->cursor[SP_KNOT_STATE_DRAGGING]) {
-			gdk_cursor_destroy (knot->cursor[SP_KNOT_STATE_DRAGGING]);
+			//gdk_cursor_destroy (knot->cursor[SP_KNOT_STATE_DRAGGING]);
 		}
 		knot->cursor[SP_KNOT_STATE_DRAGGING] = cursor;
 		break;
+	case ARG_PIXBUF:
+	        knot->pixbuf = GTK_VALUE_POINTER (*arg);
+	        break;
 	default:
 		g_assert_not_reached ();
 		break;
@@ -318,10 +335,12 @@ sp_knot_handler (GnomeCanvasItem * item, GdkEvent * event, gpointer data)
 			knot->hx = p.x - knot->x;
 			knot->hy = p.y - knot->y;
 #ifndef KNOT_NOGRAB
+			gdk_pointer_ungrab (event->button.time);
 			gnome_canvas_item_grab (knot->item,
-				GDK_POINTER_MOTION_MASK | GDK_BUTTON_RELEASE_MASK,
-				knot->cursor[SP_KNOT_STATE_DRAGGING],
-				event->button.time);
+						GDK_POINTER_MOTION_MASK | GDK_BUTTON_RELEASE_MASK,
+						knot->cursor[SP_KNOT_STATE_DRAGGING],
+						event->button.time);
+			
 #endif
 			sp_knot_set_flag (knot, SP_KNOT_GRABBED, TRUE);
 			grabbed = TRUE;
@@ -333,6 +352,14 @@ sp_knot_handler (GnomeCanvasItem * item, GdkEvent * event, gpointer data)
 			sp_knot_set_flag (knot, SP_KNOT_GRABBED, FALSE);
 #ifndef KNOT_NOGRAB
 			gnome_canvas_item_ungrab (knot->item, event->button.time);
+			gdk_pointer_grab (knot->item->canvas->layout.bin_window,
+					  FALSE,
+					  GDK_POINTER_MOTION_MASK | GDK_BUTTON_RELEASE_MASK | 
+					  GDK_BUTTON_PRESS_MASK | GDK_LEAVE_NOTIFY_MASK,
+					  NULL,
+					  knot->cursor[SP_KNOT_STATE_MOUSEOVER],
+					  event->button.time);
+
 #endif
 			if (moved) {
 				sp_knot_set_flag (knot,
@@ -373,10 +400,19 @@ sp_knot_handler (GnomeCanvasItem * item, GdkEvent * event, gpointer data)
 		}
 		break;
 	case GDK_ENTER_NOTIFY:
+		gdk_pointer_grab (knot->item->canvas->layout.bin_window,
+				  FALSE,
+				  GDK_POINTER_MOTION_MASK | GDK_BUTTON_RELEASE_MASK | 
+				  GDK_BUTTON_PRESS_MASK | GDK_LEAVE_NOTIFY_MASK,
+				  NULL,
+				  knot->cursor[SP_KNOT_STATE_MOUSEOVER],
+				  event->button.time);
+
 		sp_knot_set_flag (knot, SP_KNOT_MOUSEOVER, TRUE);
 		consumed = TRUE;
 		break;
 	case GDK_LEAVE_NOTIFY:
+		gdk_pointer_ungrab (event->button.time);
 		sp_knot_set_flag (knot, SP_KNOT_MOUSEOVER, FALSE);
 		consumed = TRUE;
 		break;
@@ -457,13 +493,6 @@ sp_knot_request_position (SPKnot * knot, ArtPoint * p, guint state)
 	/* If user did not complete, we simply move knot to new position */
 
 	if (!done) {
-#if 0
-		/* Try the logic */
-		GSList * l;
-		for (l = knot->desktop->namedview->hguides; l != NULL; l = l->next) {
-			if (fabs (p->y - SP_GUIDE (l->data)->position) < 10.0) p->y = SP_GUIDE (l->data)->position;
-		}
-#endif
 		sp_knot_set_position (knot, p, state);
 	}
 }
@@ -525,6 +554,12 @@ sp_knot_set_flag (SPKnot * knot, guint flag, gboolean set)
 	g_assert (knot != NULL);
 	g_assert (SP_IS_KNOT (knot));
 
+	if (set) {
+		knot->flags |= flag;
+	} else {
+		knot->flags &= ~flag;
+	}
+
 	switch (flag) {
 	case SP_KNOT_VISIBLE:
 		if (set) {
@@ -534,46 +569,8 @@ sp_knot_set_flag (SPKnot * knot, guint flag, gboolean set)
 		}
 		break;
 	case SP_KNOT_MOUSEOVER:
-		if (!(knot->flags & SP_KNOT_DRAGGING)) {
-			gtk_object_set (GTK_OBJECT (knot->item),
-				"fill_color",
-				knot->fill [set ? SP_KNOT_STATE_MOUSEOVER : SP_KNOT_STATE_NORMAL],
-				NULL);
-			gtk_object_set (GTK_OBJECT (knot->item),
-				"stroke_color",
-				knot->stroke [set ? SP_KNOT_STATE_MOUSEOVER : SP_KNOT_STATE_NORMAL],
-				NULL);
-		}
-		break;
 	case SP_KNOT_DRAGGING:
-		if (set) {
-			gtk_object_set (GTK_OBJECT (knot->item),
-				"fill_color",
-				knot->fill [SP_KNOT_STATE_DRAGGING],
-				NULL);
-			gtk_object_set (GTK_OBJECT (knot->item),
-				"stroke_color",
-				knot->stroke [SP_KNOT_STATE_DRAGGING],
-				NULL);
-		} else if (knot->flags & SP_KNOT_MOUSEOVER) {
-			gtk_object_set (GTK_OBJECT (knot->item),
-				"fill_color",
-				knot->fill [SP_KNOT_STATE_MOUSEOVER],
-				NULL);
-			gtk_object_set (GTK_OBJECT (knot->item),
-				"stroke_color",
-				knot->stroke [SP_KNOT_STATE_MOUSEOVER],
-				NULL);
-		} else {
-			gtk_object_set (GTK_OBJECT (knot->item),
-				"fill_color",
-				knot->fill [SP_KNOT_STATE_NORMAL],
-				NULL);
-			gtk_object_set (GTK_OBJECT (knot->item),
-				"stroke_color",
-				knot->stroke [SP_KNOT_STATE_NORMAL],
-				NULL);
-		}
+		sp_knot_set_ctrl_state (knot);
 		break;
 	case SP_KNOT_GRABBED:
 		break;
@@ -581,12 +578,6 @@ sp_knot_set_flag (SPKnot * knot, guint flag, gboolean set)
 		g_assert_not_reached ();
 		break;
 	}
-
-	if (set) {
-		knot->flags |= flag;
-	} else {
-		knot->flags &= ~flag;
-	}
 }
 
 static void
@@ -594,9 +585,18 @@ sp_knot_update_ctrl (SPKnot * knot)
 {
 	if (!knot->item) return;
 
+	gtk_object_set (GTK_OBJECT (knot->item), "shape", knot->shape, NULL);
+	gtk_object_set (GTK_OBJECT (knot->item), "mode", knot->mode, NULL);
 	gtk_object_set (GTK_OBJECT (knot->item), "size", (gdouble) knot->size, NULL);
 	gtk_object_set (GTK_OBJECT (knot->item), "anchor", knot->anchor, NULL);
+	if (knot->pixbuf) gtk_object_set (GTK_OBJECT (knot->item), "pixbuf", knot->pixbuf, NULL);
 
+	sp_knot_set_ctrl_state (knot);
+}
+
+static void
+sp_knot_set_ctrl_state (SPKnot * knot)
+{
 	if (knot->flags & SP_KNOT_DRAGGING) {
 		gtk_object_set (GTK_OBJECT (knot->item),
 				"fill_color",
