@@ -5,10 +5,7 @@
  *
  */
 
-#ifdef HAVE_CONFIG_H
-#  include <config.h>
-#endif
-
+#include <config.h>
 #include <locale.h>
 
 #include <gnome.h>
@@ -29,14 +26,20 @@
 #include "bonobo/svg-doc-factory.h"
 #endif
 
+#include "document.h"
 #include "file.h"
 #include "toolbox.h"
 #include "interface.h"
 
-enum {SP_ARG_NONE, SP_ARG_NOGUI, SP_ARG_FILE, SP_ARG_PRINT, SP_ARG_LAST};
+/* fixme: These are required for temporary slideshow hack */
+#include "desktop.h"
+#include "slide-context.h"
+
+enum {SP_ARG_NONE, SP_ARG_NOGUI, SP_ARG_FILE, SP_ARG_PRINT, SP_ARG_SLIDESHOW, SP_ARG_LAST};
 
 static GSList * sp_process_args (poptContext ctx);
 gchar * sp_global_printer = NULL;
+gboolean sp_global_slideshow = FALSE;
 
 struct poptOption options[] = {
   {
@@ -66,6 +69,15 @@ struct poptOption options[] = {
         N_("Print files to specified output file (use '| program' for pipe)"),
         N_("FILENAME")
   },
+  {
+        "slideshow",
+        's',
+        POPT_ARG_NONE,
+        &sp_global_slideshow,
+        SP_ARG_SLIDESHOW,
+        N_("Show given files one-by-one, switch to next on any key/mouse event"),
+        NULL
+  },
   { NULL, '\0', 0, NULL, 0, NULL, NULL }
 };
 
@@ -272,23 +284,60 @@ main (int argc, char *argv[])
 		}
 #endif
 
-		sodipodi = sodipodi_application_new ();
-		sodipodi_load_preferences (sodipodi);
-		gtk_signal_connect (GTK_OBJECT (sodipodi), "destroy",
-				    GTK_SIGNAL_FUNC (main_save_preferences), NULL);
-		sp_maintoolbox_create ();
-		sodipodi_unref ();
+		if (!sp_global_slideshow) {
+			sodipodi = sodipodi_application_new ();
+			sodipodi_load_preferences (sodipodi);
+			gtk_signal_connect (GTK_OBJECT (sodipodi), "destroy",
+					    GTK_SIGNAL_FUNC (main_save_preferences), NULL);
+			sp_maintoolbox_create ();
+			sodipodi_unref ();
 
-		while (fl) {
-			SPDocument * doc;
-			SPDesktop * dt;
-			doc = sp_document_new ((const gchar *) fl->data);
-			if (doc) {
-				dt = sp_desktop_new (doc, sp_document_namedview (doc, NULL));
-				sp_document_unref (doc);
-				if (dt) sp_create_window (dt, TRUE);
+			while (fl) {
+				SPDocument * doc;
+				SPViewWidget *dtw;
+				doc = sp_document_new ((const gchar *) fl->data);
+				if (doc) {
+					dtw = sp_desktop_widget_new (sp_document_namedview (doc, NULL));
+					sp_document_unref (doc);
+					if (dtw) sp_create_window (dtw, TRUE);
+				}
+				fl = g_slist_remove (fl, fl->data);
 			}
-			fl = g_slist_remove (fl, fl->data);
+		} else {
+			GSList *slides = NULL;
+			SPDocument * doc;
+			SPViewWidget *dtw;
+			/* fixme: This is terrible hack */
+			sodipodi = sodipodi_application_new ();
+			sodipodi_load_preferences (sodipodi);
+			gtk_signal_connect (GTK_OBJECT (sodipodi), "destroy",
+					    GTK_SIGNAL_FUNC (main_save_preferences), NULL);
+			sp_maintoolbox_create ();
+			sodipodi_unref ();
+
+			while (fl) {
+				doc = sp_document_new ((const gchar *) fl->data);
+				if (doc) {
+					slides = g_slist_append (slides, doc);
+				}
+				fl = g_slist_remove (fl, fl->data);
+			}
+
+			if (slides) {
+				doc = slides->data;
+				slides = g_slist_remove (slides, doc);
+				dtw = sp_desktop_widget_new (sp_document_namedview (doc, NULL));
+				if (dtw) {
+					sp_desktop_set_event_context (SP_DESKTOP_WIDGET (dtw)->desktop, SP_TYPE_SLIDE_CONTEXT);
+					gtk_object_set_data (GTK_OBJECT (SP_DESKTOP_WIDGET (dtw)->desktop), "slides", slides);
+					sp_create_window (dtw, FALSE);
+				}
+#if 0
+				sp_document_unref (doc);
+#endif
+			}
+
+			sodipodi_unref ();
 		}
 
 #ifdef ENABLE_BONOBO
@@ -316,9 +365,8 @@ static GSList *
 sp_process_args (poptContext ctx)
 {
 	GSList * fl;
-	const gchar * fn;
 	const gchar ** args;
-	gint i, a;
+	gint i;
 
 	fl = NULL;
 
