@@ -9,14 +9,19 @@
  *
  */
 
+#include <config.h>
 #include <string.h>
 #include <sys/stat.h>
+#include <time.h>
 #include <fcntl.h>
 #include <unistd.h>
+#include <signal.h>
 #include <libgnome/gnome-defs.h>
 #include <libgnome/gnome-util.h>
 #include <libgnome/gnome-i18n.h>
 #include <libgnomeui/gnome-messagebox.h>
+#include <xml/repr-private.h>
+#include "document.h"
 #include "desktop.h"
 #include "desktop-handles.h"
 #include "selection.h"
@@ -26,6 +31,7 @@
 #include "preferences-skeleton.h"
 
 enum {
+	MODIFY_SELECTION,
 	CHANGE_SELECTION,
 	SET_SELECTION,
 	SET_EVENTCONTEXT,
@@ -42,13 +48,16 @@ static void sodipodi_class_init (SodipodiClass * klass);
 static void sodipodi_init (SPObject * object);
 static void sodipodi_destroy (GtkObject * object);
 
+static void sodipodi_activate_desktop_private (Sodipodi *sodipodi, SPDesktop *desktop);
+static void sodipodi_desactivate_desktop_private (Sodipodi *sodipodi, SPDesktop *desktop);
+
 static void sodipodi_init_preferences (Sodipodi * sodipodi);
 
 struct _Sodipodi {
 	GtkObject object;
-	SPReprDoc * preferences;
-	GSList * documents;
-	GSList * desktops;
+	SPReprDoc *preferences;
+	GSList *documents;
+	GSList *desktops;
 };
 
 struct _SodipodiClass {
@@ -57,6 +66,7 @@ struct _SodipodiClass {
 	/* Signals */
 
 	void (* change_selection) (Sodipodi * sodipodi, SPSelection * selection);
+	void (* modify_selection) (Sodipodi * sodipodi, SPSelection * selection, guint flags);
 	void (* set_selection) (Sodipodi * sodipodi, SPSelection * selection);
 	void (* set_eventcontext) (Sodipodi * sodipodi, SPEventContext * eventcontext);
 	void (* new_desktop) (Sodipodi * sodipodi, SPDesktop * desktop);
@@ -70,6 +80,8 @@ static guint sodipodi_signals[LAST_SIGNAL] = {0};
 
 Sodipodi * sodipodi = NULL;
 
+static void (* segv_handler) (int) = NULL;
+
 GtkType
 sodipodi_get_type (void)
 {
@@ -98,6 +110,13 @@ sodipodi_class_init (SodipodiClass * klass)
 
 	parent_class = gtk_type_class (gtk_object_get_type ());
 
+	sodipodi_signals[MODIFY_SELECTION] = gtk_signal_new ("modify_selection",
+		GTK_RUN_LAST,
+		object_class->type,
+		GTK_SIGNAL_OFFSET (SodipodiClass, modify_selection),
+		gtk_marshal_NONE__POINTER_UINT,
+		GTK_TYPE_NONE, 1,
+		GTK_TYPE_POINTER, GTK_TYPE_UINT);
 	sodipodi_signals[CHANGE_SELECTION] = gtk_signal_new ("change_selection",
 		GTK_RUN_LAST,
 		object_class->type,
@@ -151,6 +170,9 @@ sodipodi_class_init (SodipodiClass * klass)
 	gtk_object_class_add_signals (object_class, sodipodi_signals, LAST_SIGNAL);
 
 	object_class->destroy = sodipodi_destroy;
+
+	klass->activate_desktop = sodipodi_activate_desktop_private;
+	klass->desactivate_desktop = sodipodi_desactivate_desktop_private;
 }
 
 static void
@@ -179,7 +201,7 @@ sodipodi_destroy (GtkObject * object)
 		SPDocument * document;
 		document = (SPDocument *) sodipodi->documents->data;
 		gtk_object_destroy (GTK_OBJECT (document));
-		sodipodi->documents = g_slist_remove_link (sodipodi->documents, sodipodi->documents);
+		sodipodi->documents = g_slist_remove (sodipodi->documents, sodipodi->documents->data);
 	}
 
 	g_assert (!sodipodi->documents);
@@ -196,11 +218,84 @@ sodipodi_destroy (GtkObject * object)
 	gtk_main_quit ();
 }
 
+static void
+sodipodi_activate_desktop_private (Sodipodi *sodipodi, SPDesktop *desktop)
+{
+	sp_desktop_set_active (desktop, TRUE);
+}
+
+static void
+sodipodi_desactivate_desktop_private (Sodipodi *sodipodi, SPDesktop *desktop)
+{
+	sp_desktop_set_active (desktop, FALSE);
+}
+
+/* fixme: This is EVIL, and belongs to main after all */
+
+static void
+sodipodi_segv_handler (int signum)
+{
+	static gint recursion = FALSE;
+	GSList *l;
+	gchar *home;
+	gint count, date;
+
+	if (recursion) abort ();
+	recursion = TRUE;
+
+	g_warning ("Emergency save activated");
+
+	home = g_get_home_dir ();
+	date = time (NULL);
+
+	count = 0;
+	for (l = sodipodi->documents; l != NULL; l = l->next) {
+		SPDocument *doc;
+		SPRepr *repr;
+		doc = (SPDocument *) l->data;
+		repr = sp_document_repr_root (doc);
+		if (sp_repr_attr (repr, "sodipodi:modified")) {
+			gchar *path;
+			FILE *file;
+			path = g_strdup_printf ("%s/.sodipodi/emergency-%d-%d.svg", home, date, count);
+			file = fopen (path, "w");
+			g_free (path);
+			if (!file) {
+				path = g_strdup_printf ("%s/sodipodi-emergency-%d-%d.svg", home, date, count);
+				file = fopen (path, "w");
+				g_free (path);
+			}
+			if (!file) {
+				path = g_strdup_printf ("/tmp/sodipodi-emergency-%d-%d.svg", date, count);
+				file = fopen (path, "w");
+				g_free (path);
+			}
+			if (file) {
+				sp_repr_save_stream (sp_repr_document (repr), file);
+				fclose (file);
+			}
+			count++;
+		}
+	}
+
+	g_warning ("Emergency save completed, now crashing...");
+
+	(* segv_handler) (signum);
+}
+
 Sodipodi *
 sodipodi_application_new (void)
 {
-	return gtk_type_new (SP_TYPE_SODIPODI);
+	Sodipodi *sp;
+
+	sp = gtk_type_new (SP_TYPE_SODIPODI);
 	/* fixme: load application defaults */
+
+	segv_handler = signal (SIGSEGV, sodipodi_segv_handler);
+	signal (SIGFPE, sodipodi_segv_handler);
+	signal (SIGILL, sodipodi_segv_handler);
+
+	return sp;
 }
 
 /* Preference management */
@@ -264,9 +359,8 @@ sodipodi_load_preferences (Sodipodi * sodipodi)
 		return;
 	}
 
-	/* fixme: merge preferences */
-	sp_repr_document_unref (sodipodi->preferences);
-	sodipodi->preferences = doc;
+	sp_repr_document_overwrite (sodipodi->preferences, doc, "id");
+	sp_repr_document_unref (doc);
 	g_free (fn);
 }
 
@@ -298,23 +392,17 @@ sodipodi_get_repr (Sodipodi * sodipodi, const gchar * key)
 	s = key;
 	while ((s) && (*s)) {
 		SPRepr * child;
-		const GList * l;
 		/* Find next name */
 		if ((e = strchr (s, '.'))) {
 			len = e++ - s;
 		} else {
 			len = strlen (s);
 		}
-
-		child = NULL;
-		l = sp_repr_children (repr);
-		while (l != NULL) {
-			child = (SPRepr *) l->data;
+		for (child = repr->children; child != NULL; child = child->next) {
 			id = sp_repr_attr (child, "id");
 			if ((id) && (strlen (id) == len) && (!strncmp (id, s, len))) break;
-			l = l->next;
 		}
-		if (l == NULL) return NULL;
+		if (child == NULL) return NULL;
 
 		repr = child;
 		s = e;
@@ -322,6 +410,19 @@ sodipodi_get_repr (Sodipodi * sodipodi, const gchar * key)
 	return repr;
 }
 
+void
+sodipodi_selection_modified (SPSelection *selection)
+{
+	g_return_if_fail (sodipodi != NULL);
+	g_return_if_fail (selection != NULL);
+	g_return_if_fail (SP_IS_SELECTION (selection));
+
+	if (DESKTOP_IS_ACTIVE (selection->desktop)) {
+		gtk_signal_emit (GTK_OBJECT (sodipodi), sodipodi_signals[MODIFY_SELECTION], selection,
+				 SP_OBJECT_MODIFIED_FLAG | SP_OBJECT_CHILD_MODIFIED_FLAG | SP_OBJECT_PARENT_MODIFIED_FLAG);
+	}
+}
+
 void
 sodipodi_selection_changed (SPSelection * selection)
 {
@@ -405,9 +506,10 @@ sodipodi_remove_desktop (SPDesktop * desktop)
 			gtk_signal_emit (GTK_OBJECT (sodipodi), sodipodi_signals[SET_SELECTION], NULL);
 			gtk_signal_emit (GTK_OBJECT (sodipodi), sodipodi_signals[CHANGE_SELECTION], NULL);
 		}
-		gtk_signal_emit (GTK_OBJECT (sodipodi), sodipodi_signals[DESTROY_DESKTOP], desktop);
 	}
 
+	gtk_signal_emit (GTK_OBJECT (sodipodi), sodipodi_signals[DESTROY_DESKTOP], desktop);
+
 	sodipodi->desktops = g_slist_remove (sodipodi->desktops, desktop);
 }
 
@@ -437,6 +539,32 @@ sodipodi_activate_desktop (SPDesktop * desktop)
 	gtk_signal_emit (GTK_OBJECT (sodipodi), sodipodi_signals[CHANGE_SELECTION], SP_DT_SELECTION (desktop));
 }
 
+/* fixme: These need probably signals too */
+
+void
+sodipodi_add_document (SPDocument *document)
+{
+	g_return_if_fail (sodipodi != NULL);
+	g_return_if_fail (document != NULL);
+	g_return_if_fail (SP_IS_DOCUMENT (document));
+
+	g_assert (!g_slist_find (sodipodi->documents, document));
+
+	sodipodi->documents = g_slist_append (sodipodi->documents, document);
+}
+
+void
+sodipodi_remove_document (SPDocument *document)
+{
+	g_return_if_fail (sodipodi != NULL);
+	g_return_if_fail (document != NULL);
+	g_return_if_fail (SP_IS_DOCUMENT (document));
+
+	g_assert (g_slist_find (sodipodi->documents, document));
+
+	sodipodi->documents = g_slist_remove (sodipodi->documents, document);
+}
+
 SPDesktop *
 sodipodi_active_desktop (void)
 {
@@ -530,7 +658,25 @@ sodipodi_init_preferences (Sodipodi * sodipodi)
 	close (fh);
 }
 
+void
+sodipodi_refresh_display (Sodipodi *sodipodi)
+{
+	GSList *l;
 
+	for (l = sodipodi->desktops; l != NULL; l = l->next) {
+		sp_view_request_redraw (SP_VIEW (l->data));
+	}
+}
 
+/*
+ * fixme: This also
+ */
 
+void
+sodipodi_exit (Sodipodi *sodipodi)
+{
+	/* fixme: fixme: fixme: This has to allow saving data */
+
+	gtk_main_quit ();
+}
 
