@@ -1,9 +1,19 @@
-#ifndef SP_SHAPE_H
-#define SP_SHAPE_H
+#ifndef __SP_SHAPE_H__
+#define __SP_SHAPE_H__
+
+/*
+ * SPShape
+ *
+ * Author:
+ *   Lauris Kaplinski <lauris@ximian.com>
+ *
+ * Copyright (C) 1999-2000 Lauris Kaplinski
+ * Copyright (C) 2000-2001 Ximian, Inc.
+ *
+ * Released under GNU GPL
+ */
 
 #include <libart_lgpl/art_bpath.h>
-#include "display/fill.h"
-#include "display/stroke.h"
 #include "sp-path.h"
 
 #define SP_TYPE_SHAPE            (sp_shape_get_type ())
@@ -14,18 +24,22 @@
 
 struct _SPShape {
 	SPPath path;
-	SPFill * fill;
-	SPStroke * stroke;
 };
 
 struct _SPShapeClass {
 	SPPathClass parent_class;
+
+	/* build bpath from extra shape attributes */
+	void	(* set_shape) (SPShape	*shape);
 };
 
 
 /* Standard Gtk function */
 GtkType sp_shape_get_type (void);
 
+/* Method */
+void	sp_shape_set_shape (SPShape *shape);
+
 /* Utility */
 
 #if 0
@@ -38,8 +52,4 @@ void sp_shape_set_bpath (SPShape * shape, ArtBpath * bpath, double affine[]);
 void sp_shape_set_bpath_identity (SPShape * shape, ArtBpath * bpath);
 #endif
 
-#if 1
-void sp_shape_set_stroke (SPShape * shape, SPStroke * stroke);
-#endif
-
 #endif
