@@ -1,6 +1,7 @@
 #define SP_USE_C
 
-#include "display/canvas-bgroup.h"
+#include <string.h>
+#include "display/nr-arena-group.h"
 #include "document.h"
 #include "sp-object-repr.h"
 #include "sp-use.h"
@@ -21,9 +22,8 @@ static void sp_use_read_attr (SPObject * object, const gchar * attr);
 static void sp_use_bbox (SPItem * item, ArtDRect * bbox);
 static void sp_use_print (SPItem * item, GnomePrintContext * gpc);
 static gchar * sp_use_description (SPItem * item);
-static GnomeCanvasItem * sp_use_show (SPItem * item, GnomeCanvasGroup * canvas_group, gpointer handler);
-static void sp_use_hide (SPItem * item, GnomeCanvas * canvas);
-static gboolean sp_use_paint (SPItem * item, ArtPixBuf * buf, gdouble affine[]);
+static NRArenaItem *sp_use_show (SPItem *item, NRArena *arena);
+static void sp_use_hide (SPItem *item, NRArena *arena);
 
 static void sp_use_changed (SPUse * use);
 static void sp_use_href_changed (SPUse * use);
@@ -79,7 +79,6 @@ sp_use_class_init (SPUseClass *class)
 	item_class->print = sp_use_print;
 	item_class->show = sp_use_show;
 	item_class->hide = sp_use_hide;
-	item_class->paint = sp_use_paint;
 }
 
 static void
@@ -168,31 +167,25 @@ sp_use_build (SPObject * object, SPDocument * document, SPRepr * repr)
 	sp_use_read_attr (object, "y");
 	sp_use_read_attr (object, "width");
 	sp_use_read_attr (object, "height");
-	sp_use_read_attr (object, "href");
+	sp_use_read_attr (object, "xlink:href");
 
-#if 0
 	if (use->href) {
-		SPObject * refobj;
+		SPObject *refobj;
 		refobj = sp_document_lookup_id (document, use->href);
 		if (refobj) {
-			SPRepr * childrepr;
-			const gchar * name;
+			SPRepr *childrepr;
 			GtkType type;
-			childrepr = refobj->repr;
-			name = sp_repr_name (childrepr);
-			g_assert (name != NULL);
-			type = sp_object_type_lookup (name);
+			childrepr = SP_OBJECT_REPR (refobj);
+			type = sp_repr_type_lookup (childrepr);
 			g_return_if_fail (type > GTK_TYPE_NONE);
 			if (gtk_type_is_a (type, SP_TYPE_ITEM)) {
-				SPObject * childobj;
+				SPObject *childobj;
 				childobj = gtk_type_new (type);
-				childobj->parent = object;
-				use->child = SP_ITEM (childobj);
+				use->child = (SPItem *) sp_object_attach_reref (object, childobj, NULL);
 				sp_object_invoke_build (childobj, document, childrepr, TRUE);
 			}
 		}
 	}
-#endif
 }
 
 static void
@@ -229,7 +222,7 @@ sp_use_read_attr (SPObject * object, const gchar * attr)
 		sp_use_changed (use);
 		return;
 	}
-	if (strcmp (attr, "href") == 0) {
+	if (strcmp (attr, "xlink:href") == 0) {
 		const gchar * newref;
 		newref = sp_repr_attr (object->repr, attr);
 		if (newref) {
@@ -292,50 +285,39 @@ sp_use_description (SPItem * item)
 	return g_strdup ("Empty reference [SHOULDN'T HAPPEN]");
 }
 
-static GnomeCanvasItem *
-sp_use_show (SPItem * item, GnomeCanvasGroup * canvas_group, gpointer handler)
+static NRArenaItem *
+sp_use_show (SPItem *item, NRArena *arena)
 {
-	SPUse * use;
-	GnomeCanvasItem * ci;
+	SPUse *use;
 
 	use = SP_USE (item);
 
 	if (use->child) {
-		ci = gnome_canvas_item_new (canvas_group, SP_TYPE_CANVAS_BGROUP, NULL);
-		SP_CANVAS_BGROUP (ci)->transparent = FALSE;
-		sp_item_show (use->child, GNOME_CANVAS_GROUP (ci), handler);
-		return ci;
+		NRArenaItem *ai, *ac;
+		ai = nr_arena_item_new (arena, NR_TYPE_ARENA_GROUP);
+		nr_arena_group_set_transparent (NR_ARENA_GROUP (ai), FALSE);
+		ac = sp_item_show (use->child, arena);
+		if (ac) {
+			nr_arena_item_add_child (ai, ac, NULL);
+			gtk_object_unref (GTK_OBJECT(ac));
+		}
+		return ai;
 	}
 		
 	return NULL;
 }
 
 static void
-sp_use_hide (SPItem * item, GnomeCanvas * canvas)
+sp_use_hide (SPItem * item, NRArena *arena)
 {
 	SPUse * use;
 
 	use = SP_USE (item);
 
-	if (use->child) sp_item_hide (use->child, canvas);
+	if (use->child) sp_item_hide (use->child, arena);
 
 	if (SP_ITEM_CLASS (parent_class)->hide)
-		(* SP_ITEM_CLASS (parent_class)->hide) (item, canvas);
-}
-
-static gboolean
-sp_use_paint (SPItem * item, ArtPixBuf * buf, gdouble affine[])
-{
-	SPUse * use;
-
-	use = SP_USE (item);
-
-	if (SP_ITEM_CLASS (parent_class)->paint)
-		(* SP_ITEM_CLASS (parent_class)->paint) (item, buf, affine);
-
-	if (use->child) return sp_item_paint (use->child, buf, affine);
-
-	return FALSE;
+		(* SP_ITEM_CLASS (parent_class)->hide) (item, arena);
 }
 
 static void
@@ -346,6 +328,10 @@ sp_use_changed (SPUse * use)
 static void
 sp_use_href_changed (SPUse * use)
 {
+	SPItem * item;
+
+	item = SP_ITEM (use);
+
 	if (use->child) {
 		SP_OBJECT (use->child)->parent = NULL;
 		gtk_object_destroy (GTK_OBJECT (use->child));
@@ -358,24 +344,24 @@ sp_use_href_changed (SPUse * use)
 		refobj = sp_document_lookup_id (SP_OBJECT (use)->document, use->href);
 		if (refobj) {
 			SPRepr * repr;
-			const gchar * name;
 			GtkType type;
 			repr = refobj->repr;
-			name = sp_repr_name (repr);
-			g_assert (name != NULL);
-			type = sp_object_type_lookup (name);
+			type = sp_repr_type_lookup (repr);
 			g_return_if_fail (type > GTK_TYPE_NONE);
 			if (gtk_type_is_a (type, SP_TYPE_ITEM)) {
 				SPObject * childobj;
-				GSList * l;
+				SPItemView * v;
 				childobj = gtk_type_new (type);
 				childobj->parent = SP_OBJECT (use);
 				use->child = SP_ITEM (childobj);
 				sp_object_invoke_build (childobj, SP_OBJECT (use)->document, repr, TRUE);
-				for (l = SP_ITEM (use)->display; l != NULL; l = l->next) {
-					sp_item_show (SP_ITEM (childobj),
-						      GNOME_CANVAS_GROUP (l->data),
-						      sp_desktop_item_handler);
+				for (v = item->display; v != NULL; v = v->next) {
+					NRArenaItem *ai;
+					ai = sp_item_show (SP_ITEM (childobj), v->arena);
+					if (ai) {
+						nr_arena_item_add_child (v->arenaitem, ai, NULL);
+						gtk_object_unref (GTK_OBJECT(ai));
+					}
 				}
 			}
 		}
