@@ -1,33 +1,26 @@
 #ifndef SP_SVG_H
 #define SP_SVG_H
 
-#include <gtk/gtk.h>
+#include <glib.h>
 #include <libart_lgpl/art_bpath.h>
 #include <libgnomeprint/gnome-font.h>
+#include "../helper/units.h"
+#include "../style.h"
+#if 0
 #include "../display/fill.h"
+#endif
 #include "../display/stroke.h"
 
-typedef enum {
-	SP_SVG_UNIT_ABSOLUTE = (1 << 4),
-	SP_SVG_UNIT_USER     = (1 << 5),
-	SP_SVG_UNIT_PIXELS   = (1 << 6),
-	SP_SVG_UNIT_PERCENT  = (1 << 7),
-	SP_SVG_UNIT_EM       = (1 << 8),
-	SP_SVG_UNIT_EX       = (1 << 9)
-} SPSVGUnit;
-
-#define SP_SVG_UNIT_MASK 0x3f0
-
 /* General CSS Properties */
 /* length */
 
-gdouble sp_svg_read_length (SPSVGUnit * unit, const gchar * str);
-gint sp_svg_write_length (gchar * buf, gint buflen, gdouble val, SPSVGUnit unit);
+gdouble sp_svg_read_length (const SPUnit **unit, const gchar *str, gdouble def);
+gint sp_svg_write_length (gchar *buf, gint buflen, gdouble val, const SPUnit *unit);
 
-gdouble sp_svg_read_percentage (const gchar * str);
+gdouble sp_svg_read_percentage (const gchar * str, gdouble def);
 gint sp_svg_write_percentage (gchar * buf, gint buflen, gdouble val);
 
-guint32 sp_svg_read_color (const gchar * str);
+guint32 sp_svg_read_color (const guchar * str, guint32 def);
 gint sp_svg_write_color (gchar * buf, gint buflen, guint32 color);
 
 gdouble * sp_svg_read_affine (gdouble affine[], const gchar * str);
@@ -38,8 +31,10 @@ gint sp_svg_write_affine (gchar * buf, gint buflen, gdouble affine[]);
 ArtBpath * sp_svg_read_path (const gchar * str);
 gchar * sp_svg_write_path (const ArtBpath * bpath);
 
+#if 0
 SPFillType sp_svg_read_fill_type (const gchar * str);
 gint sp_svg_write_fill_type (gchar * buf, gint buflen, SPFillType type, guint32 color);
+#endif
 
 SPStrokeType sp_svg_read_stroke_type (const gchar * str);
 ArtPathStrokeJoinType sp_svg_read_stroke_join (const gchar * str);
